"""
SCout | Special Character Out

Copyright 2017-2019, Marcel Schmalzl (MSc)
This code is licensed under a MIT license
You should have received a copy of the MIT licence along with this program.
If not see:
* https://github.com/holzkohlengrill/SCout/blob/master/LICENSE
* https://opensource.org/licenses/MIT
"""

__all__ = ['SCout', 'SCout_Logger']
from pypiscout.SCout import debug, info, wwarning, warning, error, header
from pypiscout.SCout_Logger import Logger
